// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package middleware

import (
	"fmt"
	"net/http"

	"hello-api/internal/logger"

	"hello-api/internal/app/parsers/devicecontext"

	"github.com/gin-gonic/gin"
)

const DeviceContextHeader = "Device-Context"

// DeviceContextMiddleware -- Gets the B64 encoded device context from the request header
func DeviceContextMiddleware(context *gin.Context) {
	contextHeader := context.GetHeader(DeviceContextHeader)
	parser := devicecontext.NewDeviceAuthResponseParser()
	deviceAuthContext, err := parser.ParseResponse(contextHeader)
	if err != nil {
		_ = context.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to parse device context header: %w", err))
		return
	}
	logger.BaseLogger().Debug(fmt.Sprintf("Device auth context -- %+v", deviceAuthContext))
	context.Set(DeviceContextHeader, deviceAuthContext)
	context.Next()
}
