// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package config

import (
	"context"
	"strings"

	"github.com/aws/aws-sdk-go-v2/feature/ec2/imds"

	"github.com/aws/aws-sdk-go-v2/aws"
	"github.com/aws/aws-sdk-go-v2/config"
	"github.com/aws/aws-sdk-go-v2/service/kinesis"
)

const azPlaceHolder = "<AZ>"

func CreateKinesisClient(conf *Config) (*kinesis.Client, error) {
	awsConfig, err := config.LoadDefaultConfig(context.TODO(),
		config.WithRegion(conf.Region),
	)
	if err != nil {
		return nil, err
	}

	options, err := getOptions(awsConfig, conf)
	if err != nil {
		return nil, err
	}
	return kinesis.NewFromConfig(awsConfig, options...), err
}

func getKinesisEndpoint(awsConfig aws.Config, conf *Config) (string, error) {
	client := imds.NewFromConfig(awsConfig)
	response, err := client.GetInstanceIdentityDocument(context.TODO(), &imds.GetInstanceIdentityDocumentInput{})
	if err != nil {
		return "", err
	}
	az := response.InstanceIdentityDocument.AvailabilityZone
	return strings.Replace(conf.AwsEndpoint, azPlaceHolder, az, 1), nil
}

func getOptions(awsConfig aws.Config, conf *Config) (opts []func(options *kinesis.Options), err error) {
	opts = make([]func(options *kinesis.Options), 0)
	kinesisEndpointContainAZ := strings.Contains(conf.AwsEndpoint, azPlaceHolder)
	kinesisEndpoint := conf.AwsEndpoint
	switch {
	case kinesisEndpointContainAZ:
		kinesisEndpoint, err = getKinesisEndpoint(awsConfig, conf)
		fallthrough
	case conf.IsLocalEnv():
		opts = []func(options2 *kinesis.Options){kinesis.WithEndpointResolver(kinesis.EndpointResolverFunc(
			func(region string, options kinesis.EndpointResolverOptions) (aws.Endpoint, error) {
				return aws.Endpoint{
					URL:           kinesisEndpoint,
					SigningRegion: conf.Region,
				}, nil
			}))}
		return opts, err
	default:
		return opts, err
	}
}
